/* Copyright (C) 2011
 *  * Corscience GmbH & Co. KG - Simon Schwarz <schwarz@corscience.de>
 *   *  - Added prep subcommand support
 *    *  - Reorganized source - modeled after powerpc version
 *     *
 *      * (C) Copyright 2002
 *       * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 *        * Marius Groeger <mgroeger@sysgo.de>
 *         *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <asm/bootm.h>
#include <exports.h>
#include <environment.h>
#include <wakeup.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/bootparam.h>
#include <platform_lib/board/pcb_mgr.h>
#include <watchdog.h>
#include <factorylib.h>
#include <gpt_parse.h>
#include <part.h>
#include <ext4fs.h>

/* Android reboot reason stored in this property */
#define METADATA_PART_NAME  "metadata"
#define REBOOT_REASON_FILE  "/bootstat/persist.sys.boot.reason"
#define REBOOT_QUIESCENT    "quiescent"

#define BOOTREASON_PRI_mask     (0x000000FF)
#define BOOTREASON_REG          STB_SW_DATA12_reg  //0xB806016C
typedef enum {
	COLD_BOOT,
	WARM_BOOT,
	WATCHDOG,
	RECOVERY,
	KERNEL_PANIC,
	REBOOT,
	SHUTDOWN,
	RTC_WAKEUP,
	REBOOT_SHELL,
	QUIESCENT
}eBootreason;
#define arg_next(p)             (char *)((unsigned long)p+strlen(p))
#define WDT_CNT_CLEAR		0xF00
extern int boot_power_on_mode;


int read_android_reboot_reason(char *buffer, int length)
{
	int ret = 0;
	unsigned int dev_index = 0;
	char dev_string[32] = {0};
	int part = -1;
	disk_partition_t metadata_ext4_info;
	block_dev_desc_t *metadata_ext4_dev_desc = NULL;
	int file_len = 0;
	int read_len = 0;


	// get metadata partition index
	ret = get_dev_index_by_name(METADATA_PART_NAME, &dev_index);
	if (ret < 0)
	{
		printf("Get %s partition block device index fail!\n", METADATA_PART_NAME);
		return -1;
	}

	// get metadata partition info
	snprintf(dev_string, sizeof(dev_string), "0:0x%x", dev_index);
	part = get_device_and_partition("mmc", dev_string,
			&metadata_ext4_dev_desc, &metadata_ext4_info, 1);
	if (part < 0)
	{
		printf("Get %s partition desc info fail!\n", METADATA_PART_NAME);
		return -1;
	}

	// set device as block device
	ext4fs_set_blk_dev(metadata_ext4_dev_desc, &metadata_ext4_info);

	// mount filesystem
	if (!ext4fs_mount(metadata_ext4_info.size))
	{
		ret = -1;
		printf("mount ext4 partition %s fail!\n", METADATA_PART_NAME);
		goto fail;
	}

	// open file and get file size
	ret = ext4fs_open(REBOOT_REASON_FILE, (loff_t *)&file_len);
	if (ret < 0)
	{
		printf("file %s not found in %s partition!\n", REBOOT_REASON_FILE, METADATA_PART_NAME);
		goto fail;
	}

	ret = ext4fs_read(buffer, 0, (loff_t)length, (loff_t *)&read_len);
	if (ret < 0)
	{
		printf("read file %s to ddr fail!\n", REBOOT_REASON_FILE);
	}
	else
	{
		ret = read_len;
	}

fail:
	ext4fs_close();
	return ret;
}

void set_boot_reason(char *kargs)
{
	extern uchar boot_ac_on;
	uchar wdt_count = 0;
	eBootreason ebr = COLD_BOOT;
	uint boot_cause = 0;
	char reboot_reason[128] = {0};

	if (boot_ac_on) {
		if (boot_mode != BOOT_RESCUE_MODE)
		{
			ebr = COLD_BOOT;
		}
		else
		{
			ebr = RECOVERY;
		}
	}
	else
	{
		if(boot_power_on_mode == BACKLIGHT_OFF)
		{
			ebr = RTC_WAKEUP;
		}
		else
		{
			ebr = WARM_BOOT;
		}
	}
	wdt_count = (rtd_inl(ISO_MISC_TCWCR_reg)>> 8)&0x7;
	if(wdt_count)
	{
		ebr = WATCHDOG;
		/* Clear watchdog counts. */
		boot_cause = rtd_inl(ISO_MISC_TCWCR_reg);
		boot_cause |= WDT_CNT_CLEAR;
		rtd_outl(ISO_MISC_TCWCR_reg, boot_cause);
	}

	boot_cause = rtd_inl(BOOTREASON_REG) & BOOTREASON_PRI_mask;
	if(boot_cause) {
		if(boot_cause == REBOOT)
			ebr = REBOOT;
		else if(boot_cause == KERNEL_PANIC)
			ebr = KERNEL_PANIC;
		else if(boot_cause == SHUTDOWN)
			ebr = SHUTDOWN;
	}

	if ((REBOOT == ebr) && (read_android_reboot_reason(reboot_reason, sizeof(reboot_reason)) > 0))
	{
		printf("Android reboot reason: %s\n", reboot_reason);
		if (strstr(reboot_reason, REBOOT_QUIESCENT) != NULL)
		{
			ebr = QUIESCENT;
		}
	}

	switch(ebr)
	{
		case COLD_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=cold ");
		break;
		case WARM_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=warm ");
		break;
		case WATCHDOG:
			sprintf(arg_next(kargs),"androidboot.bootreason=watchdog ");
		break;
		case RECOVERY:
			sprintf(arg_next(kargs),"androidboot.bootreason=recovery ");
		break;
		case KERNEL_PANIC:
			sprintf(arg_next(kargs),"androidboot.bootreason=kernel_panic ");
		break;
		case REBOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=reboot ");
		break;
		case SHUTDOWN:
			sprintf(arg_next(kargs),"androidboot.bootreason=shutdown ");
		break;
		case RTC_WAKEUP:
			sprintf(arg_next(kargs),"androidboot.bootreason=rtc androidboot.quiescent=1 ");
		break;
		case QUIESCENT:
			sprintf(arg_next(kargs),"androidboot.bootreason=warm androidboot.quiescent=1 ");
		break;
		default:
			printf("BOOT REASON UNKNOWN !!!! \n");
	}
	/* Shift current reason to previous reason bit field */
	boot_cause = rtd_inl(BOOTREASON_REG);
	rtd_outl(BOOTREASON_REG, (boot_cause << 16));
}
