/*
 * USB HOST EHCI Controller
 */
#include <common.h>
#include <malloc.h>
#include <usb.h>
#include <watchdog.h>
#include <asm-generic/errno.h>
#include <linux/compat.h>
#include <rbus/usb2_top_reg.h>
#include <rbus/sys_reg_reg.h>
#include "ehci.h"
#include "usb2-plat-config.h"
#include "ehci_reg.h"

#define USB_PHY_DELAY                         120
#define ehci_plat_dbg(fmt, args...)           printk("[ehci], dbg, " fmt, ## args)
#define ehci_plat_info(fmt, args...)          printk("[ehci], info, " fmt, ## args)
#define ehci_plat_warn(fmt, args...)          printk("[ehci], warn, " fmt, ## args)

static unsigned long USB2_VSTATUS_regs[] = {
    USB2_TOP_VSTATUS_reg,
    USB2_TOP_VSTATUS_2port_reg
};

static U2_PHY_CONFIG u2_phy_config = {
    .p_reg_table = u2_phy_setting,
    .n_reg       = sizeof(u2_phy_setting) / sizeof(U2_PHY_REGISTER),
};

extern int pcb_mgr_get_enum_info_byname(char *enum_name, unsigned long long *value);
extern int IO_Set(unsigned long long pin, int value);

int f_usb2_enabled = 0;

/*-----------------------------------------------
 * Func : _get_usb2_phy
 *
 * Desc : read usb2 phy register with UTMI
 *
 * Parm : port1     : port index (1~2)
 *        phy_reg   : phy register address
 *
 * Retn : others : phy register value
 *        -1 : failed
 *-----------------------------------------------*/
static int _get_usb2_phy(int port1, int phy_reg)
{
    int phy_value;
    unsigned long addr;

    if (port1 > ARRAY_SIZE(USB2_VSTATUS_regs) || port1 < 1) {
        ehci_plat_warn("%s failed, invalid port - %d\n", __func__, port1);
        return -1;
    }
    if (phy_reg < 0xe0 || phy_reg > 0xf7) {
        ehci_plat_warn("%s: failed, invalid phy_reg 0x%x\n", __func__, phy_reg);
        return -1;
    }

    addr = HMAC_HOST_EHCI_INSNREG05_reg;

    rtd_outl(addr, 0x00020000 | (phy_reg & 0x0f) << 8 | (0x2000 * port1));
    udelay(USB_PHY_DELAY);

    rtd_outl(addr, 0x00020000 | (phy_reg & 0xd0) << 4 | (0x2000 * port1));
    udelay(USB_PHY_DELAY);

    phy_value = rtd_inl(addr) & 0xff;

    return (phy_value & 0xff);
}


/*-----------------------------------------------
 * Func : _set_usb2_phy
 *
 * Desc : set usb2 phy register with UTMI
 *
 * Parm : port1     : port index (1~2)
 *        phy_reg   : phy register address
 *        phy_value : phy register value
 *
 * Retn : 0 : successed,
 *       -1 : failed
 *-----------------------------------------------*/
static int _set_usb2_phy(int port1, int phy_reg, int phy_value) {
    int value;
    unsigned long addr;
    unsigned long vstatus_reg = 0;

    if (port1 > ARRAY_SIZE(USB2_VSTATUS_regs) || port1 < 1) {
        ehci_plat_warn("%s failed, invalid port - %d\n", __func__, port1);
        return -1;
    }

    if (phy_reg < 0xe0 || phy_reg > 0xf7) {
        ehci_plat_warn("%s failed, invalid phy_reg 0x%x\n", __func__, phy_reg);
        return -1;
    }

    addr = HMAC_HOST_EHCI_INSNREG05_reg;
    vstatus_reg = USB2_VSTATUS_regs[port1 - 1];

    do {
        // write value
        rtd_outl(vstatus_reg, phy_value & 0xff);
        udelay(USB_PHY_DELAY);

        rtd_outl(addr, 0x00020000 | (phy_reg & 0x0f) << 8 | (0x2000 * port1));
        udelay(USB_PHY_DELAY);

        rtd_outl(addr, 0x00020000 | (phy_reg & 0xf0) << 4 | (0x2000 * port1));
        udelay(USB_PHY_DELAY);

        value = _get_usb2_phy(port1, phy_reg);

        if ((phy_value & 0xff) != value)
            ehci_plat_warn("set_usb2_phy, port %d phy_reg 0x%.2x set 0x%x get 0x%x <---- not matched\n",
                        port1, phy_reg, phy_value & 0xff, value);
        else{
            ehci_plat_dbg("set_usb2_phy, port %d phy_reg 0x%.2x set 0x%x get 0x%x\n",
                        port1, phy_reg, phy_value & 0xff, value);
        }

    } while (0); /* just write once */

    return value;
}

/*-----------------------------------------------
 * Func : _usb2_load_phy_setting
 *
 * Desc : load usb2 phy setting
 *
 * Parm : p_reg : register list
 *        n_reg : register value
 *
 * Retn : N/A
 *-----------------------------------------------*/
static void _usb2_load_phy_setting(
    U2_PHY_REGISTER*    p_reg,
    unsigned int        n_reg
    )
{
    while(n_reg--)
    {
        _set_usb2_phy(p_reg->port1, p_reg->addr, p_reg->val);
        p_reg++;
    }
}

static void ehci_usb2_load_phy_params(void)
{
    ehci_plat_dbg("---- %s start ----\n", __func__);
    _usb2_load_phy_setting(u2_phy_config.p_reg_table, u2_phy_config.n_reg);
    ehci_plat_dbg("---- %s complete ----\n", __func__);
}


/*
 * For MAC_TOP, SNPS reset & release.
 * 1: reset
 * 0: release
*/
static void snps_reset_release(void)
{
    //SNPS reset
    rtd_setbits(USB2_TOP_USB_CTR0_reg,
            USB2_TOP_USB_CTR0_usbphy_pow_p1(1) |
            USB2_TOP_USB_CTR0_usbphy_pow_p0(1) |
            USB2_TOP_USB_CTR0_usbip_port1_soft_reset(1) |
            USB2_TOP_USB_CTR0_usbip_port0_soft_reset(1));
    udelay(USB_PHY_DELAY);

    //SNPS release
    rtd_clearbits(USB2_TOP_USB_CTR0_reg,
            USB2_TOP_USB_CTR0_usbip_port1_soft_reset(1) |
            USB2_TOP_USB_CTR0_usbip_port0_soft_reset(1));
    udelay(USB_PHY_DELAY);
}


static void keep_pll_alive(void)
{
    rtd_setbits(USB2_TOP_USB_CTR0_reg, USB2_TOP_USB_CTR0_pll_alive_enable_mask);
}


static void rtd_usb_mac_init(void)
{
    /* UTMI relase */
    rtd_clearbits(USB2_TOP_RESET_UTMI_reg,
            USB2_TOP_RESET_UTMI_reset_utmi_p0_mask);         ////MAC 1 PORT 0 UTMI release
    rtd_clearbits(USB2_TOP_RESET_UTMI_2port_reg,
            USB2_TOP_RESET_UTMI_2port_reset_utmi_p1_mask);   ////MAC 1 PORT 1 UTMI release
}

static void usb2_clk_setting_off(void)
{
    /*---------------------------------------------------
    * shutdown sequence :
    *  1. mac clock off
    *  2. not release mac reset
    *  3. not release phy reset
    *---------------------------------------------------*/

    // mac clock disable
    rtd_outl(SYS_REG_SYS_CLKEN2_reg,
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_dwc_mask |
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_wrapper_mask |
                SYS_REG_SYS_CLKEN2_write_data(0));


    // mac reset
    rtd_outl(SYS_REG_SYS_SRST2_reg,
                SYS_REG_SYS_SRST2_rstn_usb2_ex_dwc_mask |
                SYS_REG_SYS_SRST2_rstn_usb2_ex_wrapper_mask |
                SYS_REG_SYS_SRST2_write_data(0));


    // phy reset
    rtd_outl(SYS_REG_SYS_SRST3_reg,
                SYS_REG_SYS_SRST3_rstn_usb2_phy_p3_mask |
                SYS_REG_SYS_SRST3_write_data(0));
}


static void usb2_clk_setting_on(void)
{
    /*---------------------------------------------------
    * init sequence :
    *  1. mac clock disable
    *  2. mac reset
    *  3. phy reset
    *  4. phy release
    *  5. mac clock enable
    *  6. mac clock disable
    *  7. mac release
    *  8. mac clock enable
    *---------------------------------------------------*/

    // clock off: USB2_MAC
    rtd_outl(SYS_REG_SYS_CLKEN2_reg,
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_dwc_mask |
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_wrapper_mask |
                SYS_REG_SYS_CLKEN2_write_data(0));
    udelay(USB_PHY_DELAY);

    // USB2_MAC Reset
    rtd_outl(SYS_REG_SYS_SRST2_reg,
                SYS_REG_SYS_SRST2_rstn_usb2_ex_dwc_mask |
                SYS_REG_SYS_SRST2_rstn_usb2_ex_wrapper_mask |
                SYS_REG_SYS_SRST2_write_data(0));
    udelay(USB_PHY_DELAY);

    // USB Phy Reset
    rtd_outl(SYS_REG_SYS_SRST3_reg,
                SYS_REG_SYS_SRST3_rstn_usb2_phy_p3_mask |
                SYS_REG_SYS_SRST3_rstn_usb2_phy_p2_mask |
                SYS_REG_SYS_SRST3_write_data(0));
    udelay(USB_PHY_DELAY);


    // Release USB Phy Reset
    rtd_outl(SYS_REG_SYS_SRST3_reg,
                SYS_REG_SYS_SRST3_rstn_usb2_phy_p3_mask |
                SYS_REG_SYS_SRST3_rstn_usb2_phy_p2_mask |
                SYS_REG_SYS_SRST3_write_data(1));
    udelay(USB_PHY_DELAY);

    // clock on : USB2_MAC
    rtd_outl(SYS_REG_SYS_CLKEN2_reg,
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_dwc_mask |
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_wrapper_mask |
                SYS_REG_SYS_CLKEN2_write_data(1));
    udelay(USB_PHY_DELAY);

    // clock off : USB2_MAC
    rtd_outl(SYS_REG_SYS_CLKEN2_reg,
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_dwc_mask |
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_wrapper_mask |
                SYS_REG_SYS_CLKEN2_write_data(0));
    udelay(USB_PHY_DELAY);

    // Release USB2_MAC Reset
    rtd_outl(SYS_REG_SYS_SRST2_reg,
                SYS_REG_SYS_SRST2_rstn_usb2_ex_dwc_mask |
                SYS_REG_SYS_SRST2_rstn_usb2_ex_wrapper_mask |
                SYS_REG_SYS_SRST2_write_data(1));
    udelay(USB_PHY_DELAY);

    // clock on : USB2_MAC
    rtd_outl(SYS_REG_SYS_CLKEN2_reg,
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_dwc_mask |
                SYS_REG_SYS_CLKEN2_clken_usb2_ex_wrapper_mask |
                SYS_REG_SYS_CLKEN2_write_data(1));
    udelay(USB_PHY_DELAY);
}


static void enable_vbus_power(void)
{
    int ret = 0;
    unsigned long long pin_vbus_ctrl = 0;

    ret = pcb_mgr_get_enum_info_byname("PIN_USB_POWER_CTRL3", &pin_vbus_ctrl);
    if (ret == 0) {
        IO_Set(pin_vbus_ctrl, 1);
    }
}

/* ===== export function ===== */
int rtk_usb2_hw_iniitalize(void)
{
    if(!f_usb2_enabled){
        usb2_clk_setting_on();
        udelay(350);
        snps_reset_release();
        udelay(350);
        keep_pll_alive();
        udelay(350);

        ehci_usb2_load_phy_params();
        udelay(350);

        /* mac setting */
        rtd_usb_mac_init();
        udelay(350);

        enable_vbus_power();
        udelay(350);

        f_usb2_enabled = 1;

    }
    return 0;
}


int rtk_usb2_hw_disable(void){
    if(f_usb2_enabled){
        printf("#@# %s(%d) usb2_clk_setting_off\n", __func__, __LINE__);
        usb2_clk_setting_off();
        f_usb2_enabled = 0;
    }

    return 0;
}